package com.brilliance.isc.funds.crtcri.register.settle;

import com.brilliance.isc.bo.Act;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.register.settle.DefaultSettleRegister;
import com.brilliance.isc.common.register.settle.ISettleRegister;
import com.brilliance.isc.common.setglg.service.SetglgService;
import com.brilliance.isc.common.setglg.vo.SetglgVo;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.crtcri.service.CrtcriTransactionServiceImpl;
import com.brilliance.isc.vo.funds.CrdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: CrtcriSettleRegister
 * @description: 汇入清算 Crtcri 结算注册
 * @author: huangshunlin
 * @create: 2024-10-16 20:27
 */
@Component("crtcri.settleRegister")
public class CrtcriSettleRegister implements ISettleRegister {

    @Autowired
    private SetglgService setglgService;

    @Autowired
    private DefaultSettleRegister defaultSettleRegister;

    @Autowired
    private SetmodService setmodService;

    @Autowired
    private PtsptaToolComponent ptsptaToolComponent;


    @Resource(name = "crtcri.transaction")
    CrtcriTransactionServiceImpl fxtsssTransactionService;

    @Override
    public void registerSettlement(SetmodVo setmodVo) {
        transRegister(setmodVo);
        defaultSettleRegister.registerSettlement(setmodVo);
    }

    /**
     * source:crtcri.0009.script
     */
    private void transRegister(SetmodVo setmodVo) {
        CrdBizInfoVo rec = (CrdBizInfoVo) setmodVo.getRec();
        String maxCur = "";
        BigDecimal maxAmt = BigDecimal.ZERO;
        if (setmodVo.getCbsMap().get("MAX") != null) {
            maxAmt = setmodVo.getCbsMap().get("MAX").getAmt();
            maxCur = setmodVo.getCbsMap().get("MAX").getCur();
        }
        setmodVo.setDocamt(maxAmt);
        SetglgVo setglg = setmodVo.getSetglg();
        setglgService.setglgClrAmount(setglg, "TRN");

        String rol = NULLSTR;
        List<PtsptaVo> ptsList = setmodVo.getPtsList();
        if (CollectionUtils.isNotEmpty(ptsList)) {
//        if (this.getCrdgrp().getAcc().isRolSet()) {
            if (ptsptaToolComponent.ptsIsRolSet(ptsList, "ACC")) {
                rol = "ACC";
            } else {
                rol = "SND";
            }
        }

        String dbtKey = setmodService.setGlgAmt(setmodVo, "TRN", rol, setmodVo.getDoccur(), maxAmt, "");
        if (!MdaUtils.isEmpty(rec.getForare())) {
//            this.getTrnmod().getTrnism().setDefDsp(rol, "D", "NOD" + CR + "VOD" + CR + "ARD");
        } else {
//            this.getTrnmod().getTrnism().setDefDsp(rol, "D", "NOD" + CR + "VOD");
        }
        Act rcvact = new Act();
        if (MdaUtils.isEmpty(rec.getRcvactInr()) && !MdaUtils.isEmpty(rec.getRcvbchinr())) {
            if (MdaUtils.compareTo(rec.getCustyp(), "1") == 0) {
                // 对私
                rcvact = fxtsssTransactionService.readClearingAct("SP-RIP", rec.getRcvbchinr(), maxCur);
            } else {
                // 对公
                rcvact = fxtsssTransactionService.readClearingAct("SP-RIC", rec.getRcvbchinr(), maxCur);
            }
        }
        String cdtKey = setglgService.setglgAddAmount(setglg, "TRN", "OWN", setmodVo.getDoccur(), maxAmt.negate(),
                "", "SPC", "A", rcvact.getExtkey(), "");
        setmodVo.setFeeclirol(rol);
        setmodService.setglgAllValDat(setmodVo, rec.getOpndat(), "D");
    }
}
