package com.brilliance.isc.funds.fxtssb.check;

import com.brilliance.isc.bo.Fxd;
import com.brilliance.isc.common.transaction.newcheck.AbstractModuleValidator;
import com.brilliance.isc.common.transaction.newcheck.ValidResult;
import com.brilliance.isc.common.transaction.newcheck.Validate;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;

/**
 * @program: isc-funds
 * @ClassName: FxtssbValidator
 * @description: 即期结汇 Fxtssb 校验
 * @author: huangshunlin
 * @create: 2024-10-08 15:38
 */
@Component
public class FxtssbValidator extends AbstractModuleValidator<FxtssbStoreBo> {


    @Override
    @PostConstruct
    public void register() {
        super.register();
    }

    @Override
    public Map<String, String> validate(FxtssbStoreBo bo) {
        Map<String, String> maps = new HashMap<>();
        maps.putAll(super.validate(bo));
        return maps;
    }


    /**
     * source:fxtssb.0017.script
     */
    @Validate(path = "fxdgrp.rec.trnman")
    public ValidResult checkFxdgrpRecTrnmanN1000(FxtssbStoreBo bo) {
        if (MdaUtils.isEmpty(bo.getFxdgrp().getRec().getTrnman())) {
            return errorMandatory();
        }
        return null;
    }

    /**
     * source:fxtssb.0018.script
     */
    @Validate(path = "fxdgrp.rec.trdint")
    public ValidResult checkFxdgrpRecTrdintN1000(FxtssbStoreBo bo) {
        Fxd rec = bo.getFxdgrp().getRec();
        if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 2, 1), "B") == 0 && MdaUtils.isEmpty(rec.getTrdint())) {
            return errorMandatory();
        }
        return null;
    }

    /**
     * source:fxtssb.0019.script
     */
    @Validate(path = "fxdgrp.rec.trdout")
    public ValidResult checkFxdgrpRecTrdoutN1000(FxtssbStoreBo bo) {
        Fxd rec = bo.getFxdgrp().getRec();
        if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 2, 1), "S") == 0
                && MdaUtils.isEmpty(rec.getTrdout())) {
            return errorMandatory();
        }
        return null;
    }

    /**
     * source:fxtssb.0020.script
     */
    @Validate(path = "fxdgrp.rec.acc")
    public ValidResult checkFxdgrpRecAccN1000(FxtssbStoreBo bo) {

        if (MdaUtils.isEmpty(bo.getFxdgrp().getRec().getAcc())) {
            return errorMandatory();
        }
        return null;

    }

    /**
     * source:fxtssb.0021.script
     */
    @Validate(path = "fxdgrp.rec.acc2")
    public ValidResult checkFxdgrpRecAcc2N1000(FxtssbStoreBo bo) {

        if (MdaUtils.isEmpty(bo.getFxdgrp().getRec().getAcc2())) {
            return errorMandatory();
        }
        return null;

    }

    /**
     * source:fxtssb.0027.script
     */
    @Validate(path = "fxdgrp.rec.dsp")
    public ValidResult checkFxdgrpRecDspN1000(FxtssbStoreBo bo) {
        if (MdaUtils.isEmpty(bo.getFxdgrp().getRec().getDsp())) {
            return errorMandatory();
        }
        return null;

    }

    /**
     * source:fxtssb.0028.script
     */
    @Validate(path = "fxdgrp.rec.dsp2")
    public ValidResult checkFxdgrpRecDsp2N1000(FxtssbStoreBo bo) {
        if (MdaUtils.isEmpty(bo.getFxdgrp().getRec().getDsp2())) {
            return errorMandatory();
        }
        return null;
    }


}
