package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.CrtcroStoreBo;
import com.brilliance.isc.funds.vo.funds.CrtcroStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: CrtcroStructMapper
 * @description: 汇出清算 Crtcro Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: huangshunlin
 * @create: 2024-10-16 20:30
 */
@Mapper(componentModel = "spring")
public interface CrtcroStructMapper extends IBaseStructMapper<CrtcroStoreBo, CrtcroStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    CrtcroStructMapper INSTANCE = Mappers.getMapper(CrtcroStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    CrtcroStoreBo voTransferToBo(CrtcroStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({

    })
    void voUpdateToBo(CrtcroStoreVo vo, @MappingTarget CrtcroStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    CrtcroStoreVo boTransferToVo(CrtcroStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(CrtcroStoreBo bo, @MappingTarget CrtcroStoreVo vo);
}

