package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.FxtsqoStoreBo;
import com.brilliance.isc.funds.vo.funds.FxtsqoStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: FxtsqoStructMapper
 * @description: 即期结售汇报价 转换Fxtsqo Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: huangshunlin
 * @create: 2024-10-18 15:27
 */
@Mapper(componentModel = "spring")
public interface FxtsqoStructMapper extends IBaseStructMapper<FxtsqoStoreBo, FxtsqoStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    FxtsqoStructMapper INSTANCE = Mappers.getMapper(FxtsqoStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    FxtsqoStoreBo voTransferToBo(FxtsqoStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    void voUpdateToBo(FxtsqoStoreVo vo, @MappingTarget FxtsqoStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    FxtsqoStoreVo boTransferToVo(FxtsqoStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(FxtsqoStoreBo bo, @MappingTarget FxtsqoStoreVo vo);
}

